C $Header: /u/gcmpack/MITgcm/pkg/hg/hg_chemistry.F,v 1.10 2010/03/16 00:14:47 jmc Exp $
C $Name: checkpoint64g $

C To calculate the chemistry of Hg species in the ocean, yxzhang, 5/30/13
C

#include "GCHEM_OPTIONS.h"
#include "HG_OPTIONS.h"
#ifdef USE_EXFIWR
#include "EXF_OPTIONS.h"
#include "SEAICE_OPTIONS.h"
#endif

CStartOfInterFace
      SUBROUTINE HG_CHEMISTRY( Ptr,
     &                         Gchem,    
     &                         bi,bj,imin,imax,jmin,jmax,
     &                         myIter,myTime,myThid)

C     /==========================================================\
C     | SUBROUTINE HG_CHEMISTRY                                  |
C     | o Calculate the changes to Hg species because            |
C     |   photo- and bio-chemical reactions, and food web        |
C     |==========================================================|
      IMPLICIT NONE

C     == GLobal variables ==
#include "SIZE.h"
#include "EEPARAMS.h"
#include "PARAMS.h"
#include "GRID.h"
#include "DYNVARS.h"
#include "PTRACERS_SIZE.h"
#include "PTRACERS_PARAMS.h"
#include "GCHEM.h"
#include "HG_SIZE.h"
#include "HG.h"
#include "chem_hg_Rates.h"
#include "chem_hg_Parameters.h"
#ifdef USE_EXFIWR
#include "SEAICE_SIZE.h"
#include "SEAICE.h"
#include "EXF_FIELDS.h"
#include "FFIELDS.h"
#endif

C     == Routine arguments ==
      INTEGER myIter
      _RL myTime
      INTEGER myThid
      _RL  Ptr  (1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr,nSx,nSy,PTRACERS_num)
      _RL  Gchem(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr,nSx,nSy,PTRACERS_num)    
#ifdef ALLOW_DIAGNOSTICS      
      _RL  hg2redn_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  hg2redb_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  hg2redp_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  hg0oxdb_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  hg0oxdp_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  hg0oxdd_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
#ifdef METHY_MODE
      _RL   hg2metb_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  mmhgdmeb_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  mmhgdmep_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  mmhgmetb_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
      _RL  dmhgdmep_diag(1-OLx:sNx+OLx,1-OLy:sNy+OLy,Nr)
#endif
      CHARACTER*8 diagname
#endif
      _RL,SAVE :: meansuncos(1-OLx:sNx+OLx,1-OLy:sNy+OLy)
      LOGICAL,SAVE :: FIRST
      DATA FIRST /.TRUE./
      INTEGER nl
      INTEGER i,j,k,l,m,bi, bj, imin, imax, jmin, jmax
      
#ifdef ALLOW_PTRACERS
#ifdef ALLOW_HG
C     == Local variables ==
      _RL radz, radz0, ecz1, ecz2
      _RL suncos
      _RL hg0_old, hg2_old, hg0_new, hg2_new
      _RL mmhg_old,dmhg_old,mmhg_new,dmhg_new
      _RL CONCN(NVAR)
      _RL concn_scale, small_num, small_num2, small_num3
      _RL inv_concn_scale, inv_concn_scale_dt
      _RL phy_comm_scale
      PARAMETER ( small_num = 1.0 _d -3 )
      PARAMETER ( small_num2 = 1.0 _d -10 )   
      PARAMETER ( small_num3 = 1.0 _d -15 )   
      PARAMETER ( concn_scale = 1.0 _d 12 )    ! transfer mol/m3 to fM
      PARAMETER ( inv_concn_scale = 1.0 _d -12 )

C     == Parameters
C     -- Radiation extinction
      _RL EC_w    ! m-1
      _RL EC_doc  ! m-1
      _RL EC_chla ! m-1
      data EC_w /0.0145 _d 0/
C     data EC_doc /0.654 _d 0/ ! from Soerensen et al. 2010, too large
      data EC_doc /0.026 _d 0/
      data EC_chla /3.1 _d 1/

C     -- Reaction base, scale factors
C     -- Inorganic species, from Soerensen 2010
      _RL k_hg2_hg0_rad  ! s-1
      _RL k_hg2_hg0_bio
      _RL k_hg0_hg2_rad
      _RL k_hg0_hg2_dak
      _RL k_hg0_hg2_bio

      _RL k_hg2_hg0_rad_base
      _RL k_hg2_hg0_bio_base
      _RL k_hg0_hg2_rad_base
      _RL k_hg0_hg2_dak_base
      _RL k_hg0_hg2_bio_base
      _RL k_hg0_hg2_che
      _RL k_hg2_red_fraction      
      data k_hg2_hg0_rad_base / 1.56d-6/
C     Sorensen 2010, based on production 
C     data k_hg2_hg0_bio_base / 4.51d-10/  s-1 (mgCm-2d-1 npp)-1
C     based on remin, through entire water column, s-1(mgCm-3d-1 fremin)-1
#ifdef ISHG2LABILE
      data k_hg2_hg0_bio_base / 2.48d-7/ !/ 6.2d-7 /
#else
      data k_hg2_hg0_bio_base / 6.2d-7 /
#endif
C     assume Hg0:Hg2 in deep ocean = 1:4      
      data k_hg0_hg2_bio_base / 9.9d-7 /
C     Soerensen uses 6.64d-6 for k_hg0_hg2_rad_base
C     4.7d-6 was used before May 26, 2019
C     changed to 8.6d-6 to reduce hg0 fraction and evasion
      data k_hg0_hg2_rad_base / 9.63d-6 /
C     Sorensen 2010, constant
C     now scaled based on monthly mean rad
C     data k_hg0_hg2_dak_base / 1d-7/
      data k_hg0_hg2_dak_base / 1.3d-9/
#ifndef VARI_RED_POOL
#ifdef ISHG2LABILE
      data k_hg2_red_fraction / 1.0d0 / !/4.0 _d -1/
#else
      data k_hg2_red_fraction /4.0 _d -1/
#endif
#endif
c     hg0 oxidize to hg2, to cancel mmhg to hg0     
      data k_hg0_hg2_che / 1.44 _d -9 /  !2.4 _d -9 / 

C     -- Methylation and demethylation
      _RL k_hg2_mmhg_bio_base
      _RL k_mmhg_hg2_rad_base
      _RL k_mmhg_hg0_bio_base
      _RL k_mmhg_hg0_che_20c
      _RL k_mmhg_hg0_che_ae
      _RL k_mmhg_hg0_che_re            
      _RL k_mmhg_dmhg_che
      _RL k_dmhg_mmhg_che
      _RL k_dmhg_mmhg_rad_base
      _RL k_rad2par
      
#ifdef METHY_MODE
      !anls data k_hg2_mmhg_bio_base / 4.0d-7 / *(1-pe)*CHL  mg/m3         !2.3d-9 to 5d-7
      !anls data k_mmhg_hg2_rad_base / 3.0d-6 / *PARvis      E/m2/d
      !anls data k_mmhg_hg0_bio_base / 4.6d-8 / *CHL         mg/m3         !4.0d-3 / 86400, (0.1-3) before vs039 =0.3d-3
      !anls data k_mmhg_dmhg_bio_base /1.0d-7 / *CHL         mg/m3
      !anls data k_dmhg_mmhg_rad_base /8.7d-9 / *PARvis      E/m2/d
      ! In MITgcm, averagely, chl (mg/m3) = 0.6 * fremin
      ! (mmolC/m3/d), 
      ! Soerensen estimates remin = (1-pe)*chl
      ! Globally, pe has a value of approximately 20%, so this
      ! means: remin = 0.8 * chl          
      !data k_rad2par /4.32d-1/  !W/m2 -> E/m2/d       

c biological methylation
c 6.8e-3 d-1 -> 7.8e-8 s-1 with fremin 0.075 mmol C/m3/d, Lehnherr 2011
c based on CLIVAR observations, it should be 8.1e-8 m3 d-1 s-1 mMol-1
      data k_hg2_mmhg_bio_base   /4.4 _d -7/ !/ 8.1 _d -8 /
c photochemical decomposition of MMHg in seawater
c 0.08-0.4 d-1 -> 9.3d-7 - 4.7d-6 s-1, Monperrus et al. 2007; Whalin et al., 2007      
c 0.2 d-1, Lehnherr 2011, with qsw as 20 W/m2, 1.16e-7 s-1/(W/m2)
c equivalent to 5e-3 m2/E, Lehnherr 2011
c parconv= 1/0.2174  !conversion from W/m2 to uEin/m2/s
c parfrac=0.4, fraction of qsw that is par
c 2e-8 was calculated based on surface MeHg~=10 fM
      data k_mmhg_hg2_rad_base /8.0 _d -8/ !/2.0 _d -7/ !/ 2.0 _d -8 /   
      !data k_mmhg_hg0_bio_base  / 2.76d-8 / ! now obselete
c chemical degradation of mmhg
c strong temperature dependence evidented by high conc. over SO
c rate at 20 degree C and active energy
c 0.36/d at 0 degree C, Lehnherr 2011
      data k_mmhg_hg0_che_20c   / 1.125 _d -8 / !/7.0 _d -8 /  !/ 3.5 _d -8 / !/ 6.9 _d -9 /
      data k_mmhg_hg0_che_ae    / 5457.3 / !/ 1.2 _d 4 / !/ 5457.3 /
      data k_mmhg_hg0_che_re    /1.7808 _d -8/                  
c biological methylation
c 0.8e-3 d-1->9.25e-9 s-1, Lehnherr 2011
      data k_mmhg_dmhg_che / 9.25 _d -9 /
      !data k_mmhg_dmhg_bio_base / 1.2 _d -7 / !/ 6.0 _d -8 /
      data k_dmhg_mmhg_rad_base / 3.76 _d -9 /
c dmhg:mmhg=1:2 Hammerschmidt 2012
      data k_dmhg_mmhg_che / 1.85 _d -8 /
#else
      data k_hg2_mmhg_bio_base  /0.0 _d 0 /      
      data k_mmhg_hg2_rad_base  /0.0 _d 0 /      
      !data k_mmhg_hg0_bio_base /0.0 _d 0/
      data k_mmhg_hg0_che_20c   /0.0 _d 0 /
      data k_mmhg_hg0_che_ae    /0.0 _d 0 /       
      data k_mmhg_dmhg_che      /0.0 _d 0/
      data k_dmhg_mmhg_rad_base /0.0 _d 0/
      data k_dmhg_mmhg_che      /0.0 _d 0/
#endif

C     ==local variables      
      _RL iceo,nppo,chlo,fremino,doco,o2o,dfeo
      _RL radswo,radswo0,meank,tco     
      _RL myTimeo
      LOGICAL success
      
C     == Start calculation
C     ==time variables
      inv_concn_scale_dt = inv_concn_scale / deltaTclock

C     == calculate monthly mean suncos
#if !defined(USE_EXFIWR) && !defined(COUPLE_GEOSChem)
      IF ( mod( myTime, 2592000. ) .lt. 1e-6 .or. FIRST) THEN
       FIRST = .FALSE.   
       nl = int(2592000 / deltaTclock)
       DO j=jMin,jMax
        DO i=iMin,iMax
         IF (HFacC(i,j,1,bi,bj).gt.0. _d 0) THEN
           meansuncos(i,j)=0. _d 0  
           DO l=1,nl
            myTimeo=myTime+dble(l-1)*deltaTclock  
            CALL hg_suncos(i,j,bi,bj,myTimeo,suncos)  
            meansuncos(i,j)=meansuncos(i,j)+suncos
           ENDDO
           meansuncos(i,j)=meansuncos(i,j)/dble(nl)   
         ENDIF                                   
        ENDDO
       ENDDO
      ENDIF
#ifdef ALLOW_DEBUG
        IF ( debugLevel.GE.debLevB ) THEN
         _BEGIN_MASTER(myThid)
         WRITE(standardMessageUnit,'(A,I10)')
      &   ' HG_CHEMISTRY Calculate mean suncos done,', myIter
         _END_MASTER(myThid)
        ENDIF
#endif /* ALLOW_DEBUG */         
#endif
      
      DO j=jMin,jMax
       DO i=iMin,iMax
       IF (HFacC(i,j,1,bi,bj).gt.0. _d 0) THEN
#ifdef USE_EXFIWR
        iceo = AREA(i,j,bi,bj)
C       radswo0 = swdown(i,j,bi,bj)*(1d0-iceo)
C       Qsw   :: Net upward shortwave radiation in W/m^2
C              Qsw = - ( downward - ice and snow absorption - reflected)
C              > 0 for decrease in theta (ocean cooling)
C              Typical range: -350 < Qsw < 0  
C       If seaice package is enabled, ice fraction is already taken into
C       account.     
        radswo0 = -Qsw(i,j,bi,bj)
        radswo = radswo0
#else
        iceo = fice(i,j,bi,bj)
C       Note: if using IGSM setting-up, the unit of radsw is ein/m2/d,
C             need to transfer to W/m2
C             1 ein/m2/d = 1e6*0.2174/86400 W/m2
C             Then, PAR to total SW radiation: 40%        
        radswo0 = max(radsw(i,j,bi,bj),0d0)*(1d0-iceo)  ! wm-2
C       impose a diurnal cycle
C       real-time radsw is passed from GEOS-Chem
#if !defined(COUPLE_GEOSChem)
        CALL hg_suncos(i,j,bi,bj,myTime,suncos)
        radswo = radswo0 * suncos / (meansuncos(i,j) + 1. _d -32)
#else
        radswo = radswo0
#endif
#endif   
        dfeo = max(dfe(i,j,bi,bj),0d0) 

C     == Radiation attenuation calculation
        ecz1 = 0d0
        ecz2 = 0d0
C     == localize variables
                          
        DO k=1,Nr
         IF (HFacC(i,j,k,bi,bj).gt.0. _d 0) THEN
C     == localize variables
c          nppo =    max(npp(i,j,k,bi,bj),0d0)   !not used any more
           fremino = max(Fremin(i,j,k,bi,bj),0d0)
           chlo =    max(chl(i,j,k,bi,bj),0d0)
           doco =    max(doc(i,j,k,bi,bj),0d0)
           o2o  =    max(o2(i,j,k,bi,bj),0d0)
           tco  =    theta(i,j,k,bi,bj)
           ecz2 = ecz1 + drF(k) * ( EC_w
     &           + EC_doc * doco * 1.2d-2   ! mmolC/m3 -> mg/L
     &           + EC_chla * chlo * 1.0d-3 ) ! mg chl/m3 -> mg/L
           meank = 1. _d 0 / (ecz2 - ecz1)
     &          * ( exp( -ecz1 ) - exp( -ecz2 ) )
           radz = radswo * meank
           radz0 = radswo0 * meank ! for dark oxidation
           ecz1 = ecz2                  

C     == Rate coefficent calculation
c     -- inorganic species
#ifdef VARI_RED_POOL
C          assume a linear relationship between the HgII reducible
C          pool and the DOC concentrations
C          based on Soerensen et al. (2013)
C          coast: DOC 110-150 uM, Hg0% = 9%
C          Bermuda: DOC 60-70 uM, Hg0% = 18%
C          reducible pool: (130 uM, 20%) (45 uM, 40%)
C          fraction(%)= -0.235*([DOC]-45)+40 
C          45 is the average surface ocean DOC, uM
#ifdef ISHG2LABILE
           k_hg2_red_fraction = 1.0d0
#else
           k_hg2_red_fraction = max((-0.235*(doco-45.0)+40.0)/100.0,0d0)
#endif
#endif

           RC3 = k_hg2_hg0_rad_base * radz * k_hg2_red_fraction
           RC4 = k_hg2_hg0_bio_base * fremino * 
     &                     k_hg2_red_fraction  ! mmolCm-3d-1
c          coast
           IF (doco .gt. 4.99 _d 2) THEN
                RC3 = RC3 * 0.5
                RC4 = RC4 * 0.5
           ENDIF     
 
           RC1 = k_hg0_hg2_rad_base * radz
           RC5 = k_hg0_hg2_bio_base * fremino  ! mmolCm-3d-1
           RC2 = k_hg0_hg2_dak_base * radz0
     &           + k_hg0_hg2_che

#ifdef METHY_MODE          
c     -- methylation/demethylation
           RC6 = k_hg2_mmhg_bio_base * fremino
c     &           * phy_comm_scale
c     &            300.0 / ( o2o + small_num )   ! anerobic condition boosts it 
c     &           (1.0 _d 0 + dfeo / 2.0e-13 )    ! iron deposition enhances methylation
c     &            k_max_fe_boost * dfeo / (dfeo + k_half_fe_boost) 
c     &           * max(2.5 _d -1, dfeo * 1.0 _d 12) 
           RC8 = k_mmhg_hg2_rad_base * radz 
c           !RC7 = k_mmhg_hg0_bio_base * fremino  ! this also happens at subsurface waters
           RC7 = k_mmhg_hg0_che_20c *
     &          exp( -k_mmhg_hg0_che_ae * ( 1.0 _d 0 / 
     &          ( 273.15 _d 0 + tco ) - 
     &            1.0 _d 0 /293.15 _d 0 ) )
C           IF (tco .LT. 3.0 _d 0) THEN
C                RC7 = 0.0 _d 0
C           ENDIF
c          Temperature dependent breakdown of MMhg, freeze below 7.5
c            degree c. The parameters are obtained from optimizing
c            matching CLIVAR data. 20170321 
C           IF (tco .gt. 7.5) THEN
C              RC7 = k_mmhg_hg0_che_re /
C     &           (-0.0895 + exp(-0.1485 * (min(tco,23d0) - 10.4373)))
C           ELSE
C              RC7 = 0.0 _d 0
C           ENDIF
           ! because [MeHg]/[HgII] is proportional to fremin
           ! cold temperature helps reserve MMHg

c     -- transfer between mmhg and dmhg                      
           RC9 = k_mmhg_dmhg_che
           RC10 = k_dmhg_mmhg_rad_base * radz
     &          + k_dmhg_mmhg_che   
#else                      
           RC6 = 0.0 _d 0
           RC7 = 0.0 _d 0
           RC8 = 0.0 _d 0
           RC9 = 0.0 _d 0
           RC10 = 0.0 _d 0
#endif
           
C     == Local concentrations
           hg0_old = max( PTR(i,j,k,bi,bj,ihg0), 0d0)
           hg2_old = max( PTR(i,j,k,bi,bj,ihg2), 0d0)
#ifdef METHY_MODE
           mmhg_old = max( PTR(i,j,k,bi,bj,immhg), 0d0)
           dmhg_old = max( PTR(i,j,k,bi,bj,idmhg), 0d0)
#else
           mmhg_old = 0.0 _d 0
           dmhg_old = 0.0 _d 0
#endif

C     == Feed the KPP solver
C     -- Tracer concentrations
           CONCN(ind_Hg2_aq) = max( hg2_old  * concn_scale, 0. _d 0 )
           CONCN(ind_Hg0_aq) = max( hg0_old  * concn_scale, 0. _d 0 )
           CONCN(ind_MMHg) = max( mmhg_old * concn_scale, 0. _d 0 )
           CONCN(ind_DMHg) = max( dmhg_old * concn_scale, 0. _d 0 )
C     -- Reaction rates, Note: they behave like tracers
           CONCN(ind_RR10) = 0d0
           CONCN(ind_RR9 ) = 0d0
           CONCN(ind_RR8 ) = 0d0
           CONCN(ind_RR7 ) = 0d0
           CONCN(ind_RR6 ) = 0d0
           CONCN(ind_RR5 ) = 0d0
           CONCN(ind_RR4 ) = 0d0
           CONCN(ind_RR3 ) = 0d0
           CONCN(ind_RR2 ) = 0d0
           CONCN(ind_RR1 ) = 0d0

           success = .false.

C      -- Safety check before calling KPP solver:
C         at least one tracer larger than 10^-8 fM
C         at least one reaction coefficient larger than 10^-32 s-1
           IF ( ( CONCN(ind_DMHg) .GT. small_num .OR.    ! at least a tracer larger than 0.001 fM
     &            CONCN(ind_MMHg) .GT. small_num .OR.
     &            CONCN(ind_Hg0_aq) .GT. small_num .OR. 
     &            CONCN(ind_Hg2_aq) .GT. small_num ) .AND.
     &          ( RC1 .GT. small_num2 .OR.             ! at least a reaction faster than 1/300 yrs 
     &            RC2 .GT. small_num2 .OR. 
     &            RC3 .GT. small_num2 .OR.
     &            RC4 .GT. small_num2 .OR.
     &            RC5 .GT. small_num2 .OR.
     &            RC6 .GT. small_num2 .OR.
     &            RC7 .GT. small_num2 .OR.
     &            RC8 .GT. small_num2 .OR.
     &            RC9 .GT. small_num2 .OR.
     &            RC10 .GT. small_num2 ) ) THEN

             CALL CHEM_HG_KPP( deltaTclock, CONCN )
           
             IF ( isnan(CONCN(ind_DMHg)) .OR.
     &            isnan(CONCN(ind_MMHg)) .OR.
     &            isnan(CONCN(ind_Hg0_aq)) .OR.
     &            isnan(CONCN(ind_Hg2_aq)) .OR.
     &            abs(CONCN(ind_DMHg)) .gt. 1 _d 6 .OR.
     &            abs(CONCN(ind_MMHg)) .gt. 1 _d 6 .OR.
     &            abs(CONCN(ind_Hg0_aq)) .gt. 1 _d 6 .OR.
     &            abs(CONCN(ind_Hg2_aq)) .gt. 1 _d 6 .OR.
     &            CONCN(ind_DMHg)   .LT. 0 _d 0 .OR.
     &            CONCN(ind_MMHg)   .LT. 0 _d 0 .OR.
     &            CONCN(ind_Hg0_aq) .LT. 0 _d 0 .OR.
     &            CONCN(ind_Hg2_aq) .LT. 0 _d 0 .OR.
     &            abs((CONCN(ind_DMHg) + CONCN(ind_MMHg) +
     &            CONCN(ind_Hg0_aq) + CONCN(ind_Hg2_aq)) *
     &            inv_concn_scale - (hg0_old + hg2_old + 
     &            mmhg_old + dmhg_old)) .GT. small_num3 ) THEN
                   dmhg_new = dmhg_old
                   mmhg_new = mmhg_old
                   hg0_new  = hg0_old
                   hg2_new  = hg2_old
             ELSE
                  dmhg_new = CONCN(ind_DMHg) * inv_concn_scale
                  mmhg_new = CONCN(ind_MMHg) * inv_concn_scale
                  hg0_new  = CONCN(ind_Hg0_aq) * inv_concn_scale
                  hg2_new  = CONCN(ind_Hg2_aq) * inv_concn_scale
                  success = .true.
             ENDIF

           ELSE
              dmhg_new = dmhg_old
              mmhg_new = mmhg_old
              hg0_new  = hg0_old
              hg2_new  = hg2_old
           ENDIF

c          mass check
c           IF ( abs(hg0_old + hg2_old + mmhg_old + dmhg_old -
c     &              hg0_new - hg2_new - mmhg_new - dmhg_new) 
c     &              .gt. 1.0 _d -15 ) THEN
c              print *, 'Mass not balanced after chemistry'
c              print *, ' Location: ',i,j,k
c              print *, ' Before: ',hg0_old, hg2_old, mmhg_old, dmhg_old
c              print *, ' After: ',hg0_new, hg2_new, mmhg_new, dmhg_new
c           ENDIF   

c           IF ( hg0_new .gt. 1 _d -7 .or. hg2_new .gt. 1 _d -7 .or.
c     &          mmhg_new .gt. 1 _d -7 .or. dmhg_new .gt. 1 _d -7 ) THEN
c              print *, 'High concentrations after chemistry'
c              print *, ' Location: ',i,j,k
c              print *, ' Before: ',hg0_old, hg2_old, mmhg_old, dmhg_old
c              print *, ' After: ',hg0_new, hg2_new, mmhg_new, dmhg_new
c           ENDIF

C Calculate diagnistics                                                
#ifdef ALLOW_DIAGNOSTICS
            IF ( success ) THEN  
            hg2redn_diag(i,j,k) = ( CONCN(ind_RR3) + CONCN(ind_RR4)   !net reduction, mol/m^3/s
     &                  - CONCN(ind_RR1) - CONCN(ind_RR2)
     &                  - CONCN(ind_RR5) )
     &                  * inv_concn_scale_dt
            hg2redb_diag(i,j,k) = CONCN(ind_RR4) * inv_concn_scale_dt
            hg2redp_diag(i,j,k) = CONCN(ind_RR3) * inv_concn_scale_dt
            hg0oxdb_diag(i,j,k) = CONCN(ind_RR5) * inv_concn_scale_dt
            hg0oxdp_diag(i,j,k) = CONCN(ind_RR1) * inv_concn_scale_dt
            hg0oxdd_diag(i,j,k) = CONCN(ind_RR2) * inv_concn_scale_dt
#ifdef METHY_MODE
            hg2metb_diag(i,j,k) = CONCN(ind_RR6) * inv_concn_scale_dt 
            mmhgdmeb_diag(i,j,k) = CONCN(ind_RR7) * inv_concn_scale_dt
            mmhgdmep_diag(i,j,k) = CONCN(ind_RR8) * inv_concn_scale_dt
            mmhgmetb_diag(i,j,k) = CONCN(ind_RR9) * inv_concn_scale_dt
            dmhgdmep_diag(i,j,k) = CONCN(ind_RR10) * inv_concn_scale_dt
#endif
            ENDIF
#endif

C     == Accumulate tendency
c           Gchem(i,j,k,bi,bj,ihg0)=Gchem(i,j,k,bi,bj,ihg0)+
c     &                             (hg0_new-hg0_old)
c     &                             /deltaTclock            
c           Gchem(i,j,k,bi,bj,ihg2)=Gchem(i,j,k,bi,bj,ihg2)+
c     &                             (hg2_new-hg2_old)
c     &                             /deltaTclock                       
            Ptr(i,j,k,bi,bj,ihg0) = hg0_new
            Ptr(i,j,k,bi,bj,ihg2) = hg2_new
#ifdef METHY_MODE           
c           Gchem(i,j,k,bi,bj,immhg)=Gchem(i,j,k,bi,bj,immhg)+
c     &                              (mmhg_new-mmhg_old)
c     &                             /deltaTclock     
c           Gchem(i,j,k,bi,bj,idmhg)=Gchem(i,j,k,bi,bj,idmhg)+
c     &                              (dmhg_new-dmhg_old)
c     &                             /deltaTclock     
            Ptr(i,j,k,bi,bj,immhg) = mmhg_new
            Ptr(i,j,k,bi,bj,idmhg) = dmhg_new
#endif

         ENDIF
        ENDDO
        
       endif
       ENDDO
      ENDDO
      
c write diagnostics
#ifdef ALLOW_DIAGNOSTICS
        WRITE(diagname,'(A8)') 'HG2REDN '
        CALL DIAGNOSTICS_FILL( hg2redn_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
        WRITE(diagname,'(A8)') 'HG2RBIO '
        CALL DIAGNOSTICS_FILL( hg2redb_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
        WRITE(diagname,'(A8)') 'HG2RPHO '
        CALL DIAGNOSTICS_FILL( hg2redp_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
        WRITE(diagname,'(A8)') 'HG0OBIO '
        CALL DIAGNOSTICS_FILL( hg0oxdb_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
        WRITE(diagname,'(A8)') 'HG0OPHO '
        CALL DIAGNOSTICS_FILL( hg0oxdp_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
        WRITE(diagname,'(A8)') 'HG0ODAK '
        CALL DIAGNOSTICS_FILL( hg0oxdd_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
#ifdef METHY_MODE
        WRITE(diagname,'(A8)') 'HG2METB '
        CALL DIAGNOSTICS_FILL( hg2metb_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
        WRITE(diagname,'(A8)') 'MMHGDMEB'
        CALL DIAGNOSTICS_FILL( mmhgdmeb_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
        WRITE(diagname,'(A8)') 'MMHGDMEP'
        CALL DIAGNOSTICS_FILL( mmhgdmep_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
        WRITE(diagname,'(A8)') 'MMHGMETB'
        CALL DIAGNOSTICS_FILL( mmhgmetb_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
        WRITE(diagname,'(A8)') 'DMHGDMEP'
        CALL DIAGNOSTICS_FILL( dmhgdmep_diag(1-Olx,1-Oly,1), diagname,
     &                         0,Nr,2,bi,bj,myThid )
#endif
#endif

#endif
#endif

       RETURN
       END
